==> field_visibility_conditions-1.0.0.tar.gz: field_visibility_conditions/src/FormAlters.php <==
<?php

declare(strict_types=1);

namespace Drupal\field_visibility_conditions;

use Drupal\Component\Plugin\ContextAwarePluginInterface;
use Drupal\Core\Condition\ConditionAccessResolverTrait;
use Drupal\Core\Condition\ConditionManager;
use Drupal\Core\Condition\ConditionPluginCollection;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityFormInterface;
use Drupal\Core\Entity\FieldableEntityInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Field\FieldConfigInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Form\SubformState;
use Drupal\Core\Plugin\Context\ContextHandlerInterface;
use Drupal\Core\Plugin\Context\ContextRepositoryInterface;
use Drupal\Core\Plugin\ContextAwarePluginAssignmentTrait;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Collection of form alters related to the field_visibility_conditions module.
 */
final class FormAlters implements ContainerInjectionInterface {

  use ConditionAccessResolverTrait;
  use ContextAwarePluginAssignmentTrait;
  use StringTranslationTrait;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $module_handler = $container->get('module_handler');
    $condition_plugin_manager = $container->get('plugin.manager.condition');
    $entity_field_manager = $container->get('entity_field.manager');
    $context_handler = $container->get('context.handler');
    $context_repository = $container->get('context.repository');

    assert($module_handler instanceof ModuleHandlerInterface);
    assert($condition_plugin_manager instanceof ConditionManager);
    assert($entity_field_manager instanceof EntityFieldManagerInterface);
    assert($context_handler instanceof ContextHandlerInterface);
    assert($context_repository instanceof ContextRepositoryInterface);

    return new self(
      $module_handler,
      $condition_plugin_manager,
      $entity_field_manager,
      $context_handler,
      $context_repository
    );
  }

  /**
   * Constructor for FormAlters.
   */
  public function __construct(
    protected readonly ModuleHandlerInterface $moduleHandler,
    protected readonly ConditionManager $conditionManager,
    protected readonly EntityFieldManagerInterface $entityFieldManager,
    protected readonly ContextHandlerInterface $contextHandler,
    protected readonly ContextRepositoryInterface $contextRepository,
    protected array $conditions = [],
  ) {
    // Get the condition plugin manager and load all the available conditions.
    $this->conditions = $this->conditionManager->getDefinitions();
  }

  /**
   * Alter the field config edit forms.
   *
   * @param array $form
   *   The form being altered.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state object.
   * @param string $form_id
   *   The ID of the form being altered.
   */
  public function fieldConfigEditFormAlter(&$form, FormStateInterface $form_state, $form_id): void {
    if (isset($form['#entity'])) {
      $form['third_party_settings']['field_visibility_conditions'] = [
        '#type' => 'details',
        '#title' => $this->t('Field Visibility'),
        '#open' => TRUE,
      ];

      // Grab the field entity from the form.
      $field_entity = $form_state->getFormObject()->getEntity();

      // Allow other modules to alter which conditions should be available.
      $this->moduleHandler->alter('field_visibility_conditions_available_conditions', $this->conditions, $form_state, $form_id);

      $contexts = $this->contextRepository->getAvailableContexts();

      // Loop through all the available conditions.
      array_walk($this->conditions, function ($condition, $id) use (&$form, $form_state, $field_entity, $contexts) {
        // Get any already saved condition settings.
        $context_conditions = $field_entity->getThirdPartySetting('field_visibility_conditions', $id) ?? [];

        // Create an instance of this condition so we can build its form.
        $instance = $this->conditionManager->createInstance($id, $context_conditions);
        $instance_form = [
          '#type' => 'details',
          '#title' => $condition['label'],
        ];
        $form['third_party_settings']['field_visibility_conditions'][$id] = [];

        // Create a subform state so we can submit this subform independently.
        $subform_state = SubformState::createForSubform($form['third_party_settings']['field_visibility_conditions'][$id], $form, $form_state);

        // Build the configuration form for the condition.
        $form['third_party_settings']['field_visibility_conditions'][$id] = $instance->buildConfigurationForm($instance_form, $subform_state);

        // Attach any applicable contexts to the form.
        $form['third_party_settings']['field_visibility_conditions'][$id]['context_mapping'] = $this->addContextAssignmentElement($instance, $contexts);
      });

      // Add our submission handler to be first.
      array_unshift($form['actions']['submit']['#submit'], [$this, 'fieldConfigEditFormSubmit']);
    }
  }

  /**
   * Submit callback for the form being submitted.
   *
   * @param array $form
   *   The form being altered.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state object.
   */
  public function fieldConfigEditFormSubmit(array &$form, FormStateInterface $form_state): void {
    // Allow other modules to alter which conditions should be available.
    $this->moduleHandler->alter('field_visibility_conditions_available_conditions', $this->conditions, $form_state, $form_id);

    // Loop through all the available conditions.
    array_walk($this->conditions, function ($condition, $id) use (&$form, $form_state) {
      // Get the submitted values for this condition.
      $context_conditions = $form_state->getValue(['third_party_settings', 'field_visibility_conditions', $id]);

      // Build an instance from the values so we can submit the subform.
      $instance = $this->conditionManager->createInstance($id, $context_conditions);

      // Build a subform state for just this condition.
      $subform_state = SubformState::createForSubform($form['third_party_settings']['field_visibility_conditions'][$id], $form, $form_state);

      // If this condition relies on contexts, grab those.
      if ($instance instanceof ContextAwarePluginInterface && $instance->getContextDefinitions()) {
        $context_mapping = $subform_state->getValue('context_mapping', []);
        $instance->setContextMapping($context_mapping);
      }

      // Submit this condition's form.
      $instance->submitConfigurationForm($form, $subform_state);

      // Set the form values to be the process instance configuration.
      $form_state->setValue(['third_party_settings', 'field_visibility_conditions', $id], $instance->getConfiguration());
    });
  }

  /**
   * Alter any fieldable entity forms to control visibility of fields.
   *
   * @param array $form
   *   The form being altered.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state object.
   * @param string $form_id
   *   The ID of the form being altered.
   */
  public function formAlter(&$form, FormStateInterface $form_state, $form_id): void {
    $form_object = $form_state->getFormObject();
    if ($form_object instanceof EntityFormInterface) {
      $entity = $form_object->getEntity() ?? [];
      if ($entity instanceof FieldableEntityInterface) {
        $field_definitions = $this->entityFieldManager->getFieldDefinitions($entity->getEntityTypeId(), $entity->bundle());
        $field_definitions = array_filter($field_definitions, function ($field) {
          return $field instanceof FieldConfigInterface;
        });

        // Allow other modules to alter which conditions should be available.
        $this->moduleHandler->alter('field_visibility_conditions_available_conditions', $this->conditions, $form_state, $form_id);

        // Loop through each field so we can check if there are conditions.
        array_walk($field_definitions, function ($field, $field_name) use (&$form) {
          $applicable_conditions = [];

          // If this field does not exist on this form.
          if (empty($form[$field_name])) {
            return;
          }

          // Loop through each condition to see if it is applicable to this
          // field.
          array_walk($this->conditions, function ($condition_object, $id) use ($field, &$applicable_conditions) {
            // Grab the condition settings for this condition.
            $condition_settings = $field->getThirdPartySetting('field_visibility_conditions', $id) ?? [];

            // If there are settings for this condition, add them to the array.
            if (!empty($condition_settings)) {
              $applicable_conditions[$id] = $condition_settings;
            }
          });

          // If there were conditions.
          if (!empty($applicable_conditions)) {
            // Create a condition collection.
            $collection = new ConditionPluginCollection($this->conditionManager, $applicable_conditions);
            $collection_array = iterator_to_array($collection);

            $conditions = [];

            // Loop through each condition, verify the contexts are available.
            array_walk($collection_array, function ($condition, $condition_id) use (&$conditions) {
              if ($condition instanceof ContextAwarePluginInterface) {
                // Try to attach any necessary contexts to the condition.
                try {
                  $contexts = $this->contextRepository->getRuntimeContexts(array_values($condition->getContextMapping()));
                  $this->contextHandler->applyContextMapping($condition, $contexts);
                }
                catch (\Exception $e) {
                  // Do nothing.
                }
              }
              $conditions[$condition_id] = $condition;
            });

            // Check the conditions.
            if ($this->resolveConditions($conditions, 'and') === FALSE) {
              // If one or more failed, block access to this field.
              $form[$field_name]['#access'] = FALSE;
            }
          }
        });
      }
    }
  }

}

==> field_visibility_conditions-1.2.0.tar.gz: field_visibility_conditions/src/FormAlters.php <==
<?php

declare(strict_types=1);

namespace Drupal\field_visibility_conditions;

use Drupal\Component\Plugin\ContextAwarePluginInterface;
use Drupal\Core\Condition\ConditionAccessResolverTrait;
use Drupal\Core\Condition\ConditionManager;
use Drupal\Core\Condition\ConditionPluginCollection;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\DependencyInjection\DependencySerializationTrait;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityFormInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Entity\FieldableEntityInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Field\FieldConfigInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Form\SubformState;
use Drupal\Core\Plugin\Context\ContextHandlerInterface;
use Drupal\Core\Plugin\Context\ContextRepositoryInterface;
use Drupal\Core\Plugin\ContextAwarePluginAssignmentTrait;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\field_visibility_conditions\Form\FieldVisibilityConditionsSettingsForm;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Constructs a new \Drupal\field_visibility_conditions\FormAlters object.
 */
final class FormAlters implements ContainerInjectionInterface {

  use ConditionAccessResolverTrait;
  use ContextAwarePluginAssignmentTrait;
  use DependencySerializationTrait;
  use StringTranslationTrait;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $module_handler = $container->get('module_handler');
    $condition_plugin_manager = $container->get('plugin.manager.condition');
    $entity_field_manager = $container->get('entity_field.manager');
    $context_handler = $container->get('context.handler');
    $context_repository = $container->get('context.repository');
    $config_factory = $container->get('config.factory');
    $entity_type_manager = $container->get('entity_type.manager');
    assert($module_handler instanceof ModuleHandlerInterface);
    assert($condition_plugin_manager instanceof ConditionManager);
    assert($entity_field_manager instanceof EntityFieldManagerInterface);
    assert($context_handler instanceof ContextHandlerInterface);
    assert($context_repository instanceof ContextRepositoryInterface);
    assert($config_factory instanceof ConfigFactoryInterface);
    assert($entity_type_manager instanceof EntityTypeManagerInterface);
    return new self(
      $module_handler,
      $condition_plugin_manager,
      $entity_field_manager,
      $context_handler,
      $context_repository,
      $config_factory,
      $entity_type_manager
    );
  }

  /**
   * Constructor for FormAlters.
   */
  public function __construct(
    protected readonly ModuleHandlerInterface $moduleHandler,
    protected readonly ConditionManager $conditionManager,
    protected readonly EntityFieldManagerInterface $entityFieldManager,
    protected readonly ContextHandlerInterface $contextHandler,
    protected readonly ContextRepositoryInterface $contextRepository,
    protected readonly ConfigFactoryInterface $configFactory,
    protected readonly EntityTypeManagerInterface $entityTypeManager,
    protected array $conditions = [],
  ) {
    // Get the condition plugin manager and load all the available conditions.
    $all_conditions = $this->conditionManager->getDefinitions();

    // Filter conditions based on admin settings.
    $enabled_conditions = $this->configFactory
      ->get(FieldVisibilityConditionsSettingsForm::SETTINGS)
      ->get('enabled_conditions') ?: [];

    // If we have enabled conditions defined, filter by them.
    if (!empty($enabled_conditions)) {
      $this->conditions = array_intersect_key($all_conditions, array_flip($enabled_conditions));
    }
    else {
      // If no settings exist yet, use all conditions.
      $this->conditions = $all_conditions;
    }
  }

  /**
   * Alter the field config edit forms.
   *
   * @param array $form
   *   The form being altered.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state object.
   * @param string $form_id
   *   The ID of the form being altered.
   */
  public function fieldConfigEditFormAlter(array &$form, FormStateInterface $form_state, string $form_id): void {
    if (isset($form['#entity'])) {
      $form['third_party_settings']['field_visibility_conditions'] = [
        '#type' => 'details',
        '#title' => $this->t('Field Visibility'),
        '#open' => TRUE,
      ];

      // Grab the field entity from the form.
      $form_object = $form_state->getFormObject();
      $field_entity = NULL;

      // First try using the form's entity property directly.
      if (isset($form['#entity']) && $form['#entity'] instanceof FieldConfigInterface) {
        $field_entity = $form['#entity'];
      }
      // Otherwise try to get the entity from the form object if it's an
      // EntityForm.
      elseif ($form_object instanceof EntityFormInterface) {
        $entity = $form_object->getEntity();
        if ($entity instanceof FieldConfigInterface) {
          $field_entity = $entity;
        }
      }

      if (!$field_entity instanceof FieldConfigInterface) {
        return;
      }

      // Allow other modules to alter which conditions should be available.
      $this->moduleHandler->alter('field_visibility_conditions_available_conditions', $this->conditions, $form_state, $form_id);

      $contexts = $this->contextRepository->getAvailableContexts();

      // Loop through all the available conditions.
      array_walk($this->conditions, function ($condition, $id) use (&$form, $form_state, $field_entity, $contexts) {
        // Get any already saved condition settings.
        $context_conditions = $field_entity->getThirdPartySetting('field_visibility_conditions', $id) ?? [];

        // Create an instance of this condition so we can build its form.
        $instance = $this->conditionManager->createInstance($id, $context_conditions);
        $instance_form = [
          '#type' => 'details',
          '#title' => $condition['label'],
        ];
        $form['third_party_settings']['field_visibility_conditions'][$id] = [];

        // Create a subform state so we can submit this subform independently.
        $subform_state = SubformState::createForSubform($form['third_party_settings']['field_visibility_conditions'][$id], $form, $form_state);

        // Build the configuration form for the condition.
        $form['third_party_settings']['field_visibility_conditions'][$id] = $instance->buildConfigurationForm($instance_form, $subform_state);

        // Attach any applicable contexts to the form.
        $form['third_party_settings']['field_visibility_conditions'][$id]['context_mapping'] = $this->addContextAssignmentElement($instance, $contexts);
      });

      // Add our submission handler to be first.
      array_unshift(
        $form['actions']['submit']['#submit'],
        [$this, 'fieldConfigEditFormSubmit']
      );
    }
  }

  /**
   * Submit callback for the form being submitted.
   *
   * @param array $form
   *   The form being altered.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state object.
   */
  public function fieldConfigEditFormSubmit(array &$form, FormStateInterface $form_state): void {
    $form_id = $form['#form_id'] ?? NULL;
    // Allow other modules to alter which conditions should be available.
    $this->moduleHandler->alter('field_visibility_conditions_available_conditions', $this->conditions, $form_state, $form_id);

    // Loop through all the available conditions.
    array_walk($this->conditions, function ($condition, $id) use (&$form, $form_state) {
      // Get the submitted values for this condition.
      $context_conditions = $form_state->getValue(
        ['third_party_settings', 'field_visibility_conditions', $id]
      );

      // Build an instance from the values so we can submit the subform.
      $instance = $this->conditionManager->createInstance($id, $context_conditions);

      // Build a subform state for just this condition.
      $subform_state = SubformState::createForSubform(
        $form['third_party_settings']['field_visibility_conditions'][$id],
        $form,
        $form_state
      );

      // If this condition relies on contexts, grab those.
      if ($instance instanceof ContextAwarePluginInterface && $instance->getContextDefinitions()) {
        $context_mapping = $subform_state->getValue('context_mapping', []);
        $instance->setContextMapping($context_mapping);
      }

      // Submit this condition's form.
      $instance->submitConfigurationForm($form, $subform_state);

      // Set the form values to be the process instance configuration.
      $form_state->setValue(
        ['third_party_settings', 'field_visibility_conditions', $id],
        $instance->getConfiguration()
      );
    });
  }

  /**
   * Alter any fieldable entity forms to control visibility of fields.
   *
   * @param array $form
   *   The form being altered.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state object.
   * @param string $form_id
   *   The ID of the form being altered.
   */
  public function formAlter(array &$form, FormStateInterface $form_state, string $form_id): void {
    $form_object = $form_state->getFormObject();
    $entity_type = NULL;
    $bundle = NULL;

    if ($form_object instanceof EntityFormInterface) {
      $entity = $form_object->getEntity() ?? [];
      $entity_type = $entity->getEntityTypeId() ?? NULL;
      $bundle = $entity->bundle() ?? NULL;
    }

    if ($this->moduleHandler->moduleExists('entity_browser')) {
      if (method_exists($form_object, 'getEntityBrowser')) {
        $entity_browser = $form_object->getEntityBrowser();
        $widgets = $entity_browser->getWidgets();
        foreach ($widgets as $widget) {
          if ($widget->getPluginId() === 'entity_form') {
            $configuration = $widget->getConfiguration();
            $entity_type = $configuration['settings']['entity_type'];
            $bundle = $configuration['settings']['bundle'];
            break;
          }
        }
      }
    }
    if ($entity_type && $bundle) {
      $fieldable_entity = $this->entityTypeManager->getDefinition($entity_type)->entityClassImplements(FieldableEntityInterface::class);
      if ($fieldable_entity) {
        $field_definitions = $this->entityFieldManager->getFieldDefinitions($entity_type, $bundle);
        $field_definitions = array_filter($field_definitions, function ($field) {
          return $field instanceof FieldConfigInterface;
        });

        // Allow other modules to alter which conditions should be available.
        $this->moduleHandler->alter('field_visibility_conditions_available_conditions', $this->conditions, $form_state, $form_id);

        // Loop through each field so we can check if there are conditions.
        array_walk($field_definitions, function ($field, $field_name) use (&$form) {
          $applicable_conditions = [];

          // If this field does not exist on this form.
          if (empty($form[$field_name])) {
            return;
          }

          // Loop through each condition to see if it is applicable to this
          // field.
          array_walk($this->conditions, function ($condition_object, $id) use ($field, &$applicable_conditions) {
            assert($field instanceof FieldConfigInterface);

            // Grab the condition settings for this condition.
            $condition_settings = $field->getThirdPartySetting('field_visibility_conditions', $id) ?? [];

            // If there are settings for this condition, add them to the array.
            if (!empty($condition_settings)) {
              $applicable_conditions[$id] = $condition_settings;
            }
          });

          // If there were conditions.
          if (!empty($applicable_conditions)) {
            // Create a condition collection.
            $collection = new ConditionPluginCollection($this->conditionManager, $applicable_conditions);
            $collection_array = iterator_to_array($collection);

            $conditions = [];

            // Loop through each condition, verify the contexts are available.
            array_walk($collection_array, function ($condition, $condition_id) use (&$conditions) {
              if ($condition instanceof ContextAwarePluginInterface) {
                // Try to attach any necessary contexts to the condition.
                try {
                  $contexts = $this->contextRepository->getRuntimeContexts(array_values($condition->getContextMapping()));
                  $this->contextHandler->applyContextMapping($condition, $contexts);
                }
                catch (\Exception $e) {
                  // Do nothing.
                }
              }
              $conditions[$condition_id] = $condition;
            });

            // Check the conditions.
            if ($this->resolveConditions($conditions, 'and') === FALSE) {
              // If one or more failed, block access to this field.
              $form[$field_name]['#access'] = FALSE;
            }
          }
        });
      }
    }
  }

}
